;***************************************************************************
;*
;* Title:		Stepper Motor Controller
;* Version:		1.0
;* Last updated:	03.19.2002
;* Target:		AT94K FPSLIC Family
;*
;* Support E-mail:	fpslic@atmel.com
;*
;* DESCRIPTION
;*	This application note describes how to implement a compact-size
;*	and high-speed interrupt driven step motor controller.
;*
;***************************************************************************

.include "at94kdef.inc"


;*************** Define Global Registers ************************
.def	rTemp	= r16


;*************** Define Constants *******************************
.equ	cValue	= 500


;***************************************************************************
;*
;*	PROGRAM START - EXECUTION STARTS HERE
;*
;***************************************************************************

.cseg

.org $0000
	rjmp	Main
.org TIM1_COMPAaddr
	rjmp	OC1A


;***************************************************************************
;* OC1A		- Timer1 Output compare A interrupt routine
;*
;* DESCRIPTION
;*
;*This interrupt routine loads new step motor value from the step
;*motor table in SRAM. The values in the table have two functions,
;*the lower nibble contains the value to output to the step motor.
;*The upper nibble holds the address of the next value. First the
;*step value is output to the port, next the address is moved to
;*the XL register.
;*
;* Number of words												:6 + return
;* Number of cycles												:7 + return
;* Low registers used												:None
;* High registers used	 :3 (temp,XL,XH)
;***************************************************************************

OC1A:
	in	rTemp, SREG
	push	rTemp
	ld	rTemp, X		; Load rTemp with X Pointer Value
	mov	XL, rTemp		; Move Value to X Pointer
	andi	rTemp, $0F		; Mask Away Upper Nibble
	out	PORTD, rTemp		; Output Lower Nibble to Stepper Motor
	swap	XL			; Swap Upper and Lower Nibble
	andi	XL, $0F			; Mask Away Upper Nibble, Address is Ready
	pop	rTemp
	out	SREG, rTemp
	reti


;***************************************************************************
;*
;* Main Program
;*
;*This program initializes Timer 1 output compare interrupt to
;*occur with a interval defined with the c_value constant.
;*The step motor lookup table is loaded from the flash and stored
;*in SRAM address 0x0100 to achieve maximum speed.
;*
;***************************************************************************

MAIN:

	ldi	rTemp, high($0FFF)     ; Initialize Stack Pointer
	out	SPH, rTemp
	ldi	rTemp, low($0FFF)
	out	SPL, rTemp
	ldi	rTemp, $0F		; Set PORTD(3..0) as Output
	out	DDRD, rTemp
	ldi	rTemp, $00
	out	PORTD, rTemp		; Write Initial Value to PORTD
	ldi	rTemp, high(cValue)	; Load Compare High Value
	out	OCR1AH, rTemp
	ldi	rTemp, low(cValue)	; Load Compare Low Value
	out	OCR1AL, rTemp
	ldi	rTemp, $00
	out	TCNT1H, rTemp		; Clear TC1 High Byte
	out	TCNT1L, rTemp		; Clear TC1 Low Byte
	out	TCCR1A, rTemp		; Clear TC1 Control Register A
	ldi	rTemp, $40
	out	TIFR, rTemp		; Clear Pending Timer Interrupt
	out	TIMSK, rTemp		; Enable TC1 Compare Interrupt

	ldi	ZH, high(step * 2)	; Init Z Pointer to Step Table in Prog Memory
	ldi	ZL, low(step * 2)
	ldi	XH, high($0100)		; Init X Pointer to Data SRAM Location
	ldi	XL, low($0100)

	ldi	rTemp, $04		; Load Counter Value
LOAD:	lpm				; Load Step Value from Prog Memory
	st	X+, r0			; Store Step Value in Data SRAM
	adiw	ZL, $01			; Increment Prog Memory Pointer
	dec	rTemp			; Decrement Counter
	brne	LOAD			; Continue until Table is Loaded

	ldi	XH, high($0100)		; Init X Pointer to Data SRAM Location
	ldi	XL, low($0100)

	ldi	rTemp, $09
	out	TCCR1B, rTemp		; Clear Timer on Compare Match, CK/1
	sei				; Enable Global Interrupts

LOOP:	rjmp	LOOP			; Do Something Else



STEP:	.db	$019, $02C, $036, $03	; Stepper Motor Look-Up Table